@@ -22,14 +22,20 @@ from torch import nn
 
 import poptorch
 from optimum.utils import logging
-from transformers import AutoConfig, PreTrainedModel
-from transformers.modeling_outputs import ModelOutput
+from transformers import PreTrainedModel
 
 from .ipu_configuration import IPUConfig
 
 
 logger = logging.get_logger(__name__)
 
+
+class IncompatibleIPUConfigError(Exception):
+    """An exception used when an IPU Config is incompatible with a model"""
+
+    pass
+
+
 _PRETRAINED_TO_PIPELINED_REGISTRY = {}
 
 
@@ -92,6 +98,7 @@ class PipelineMixin:
         pipelined_model = cls(config)
         pipelined_model.load_state_dict(model.state_dict())
         pipelined_model.ipu_config = copy.deepcopy(ipu_config)
+        pipelined_model.training = model.training
         return pipelined_model
 
     @classmethod
@@ -191,44 +198,154 @@ class PipelineMixin:
             return sum(p.numel() for p in self.parameters() if p.requires_grad or not only_trainable)
 
 
-class GenerationMethodsMixin:
-    def get_encoder(
-        self,
-        device_iterations: Optional[int] = None,
-        replication_factor: Optional[int] = None,
-        for_inference: bool = True,
-    ):
-        if not hasattr(self, "_wrapped_encoder"):
-            encoder = super().get_encoder()
-            if self.ipu_config.execute_encoder_on_cpu_for_generation:
-                self._wrapped_encoder = encoder.to(torch.float32)
-            else:
-                self.eval_opts = self.ipu_config.to_options(for_inference=True)
-                self._wrapped_encoder = poptorch.inferenceModel(
-                    encoder, options=self.ipu_config.to_options(for_inference=True)
+def _expand_layers_per_ipu_wildcard(
+    ipu_config: IPUConfig, target_number_of_layers: Optional[Union[int, List]] = None
+) -> List[int]:
+    """
+    Expand any wildcard values in `ipu_config.layers_per_ipu`.
+
+    For example, if we have:
+    ```
+    layers_per_ipu = [-1, -1]
+    target_number_of_layers = 9
+    ```
+    this function will expand the wildcard values to `layers_per_ipu = [4, 5]`
+    """
+    layers_per_ipu = copy.deepcopy(ipu_config.layers_per_ipu)
+    ipus_per_replica = ipu_config.ipus_per_replica
+
+    # Check inputs are valid
+    if not all(isinstance(n, int) and n >= -1 for n in layers_per_ipu):
+        raise IncompatibleIPUConfigError("Invalid values in layers_per_ipu. " f"layers_per_ipu={layers_per_ipu}")
+    if ipus_per_replica < 1:
+        raise IncompatibleIPUConfigError("Invalid value for ipus_per_replica. " f"ipus_per_replica={ipus_per_replica}")
+
+    if target_number_of_layers is not None:
+        if not isinstance(target_number_of_layers, int):
+            target_number_of_layers = len(target_number_of_layers)
+
+        # if ipus_per_replica is 1, then put everything on IPU0, ignoring layers_per_ipu
+        if ipus_per_replica == 1:
+            return [target_number_of_layers]
+
+        elif ipus_per_replica > 1:
+            # default/wildcards - split layers evenly over all ipus
+            if layers_per_ipu in ([-1], [-1] * ipus_per_replica):
+                quotient, remainder = divmod(target_number_of_layers, ipus_per_replica)
+                layers_per_ipu = [quotient] * ipus_per_replica
+                if remainder > 0:
+                    # add any remainder layers to last wildcard IPU
+                    layers_per_ipu[-1] += remainder
+
+            # combination of wildcards and integers
+            elif -1 in layers_per_ipu and len(layers_per_ipu) == ipus_per_replica:
+                wildcard_idxs = [idx for idx, v in enumerate(layers_per_ipu) if v == -1]
+                num_wildcard_ipus = len(wildcard_idxs)
+                # wildcard_layers = target_num_layers - num_non_wildcard_layers
+                num_wildcard_layers = target_number_of_layers - sum([l for l in layers_per_ipu if l != -1])
+                quotient, remainder = divmod(num_wildcard_layers, num_wildcard_ipus)
+                for idx in wildcard_idxs:
+                    layers_per_ipu[idx] = quotient
+                if remainder > 0:
+                    # add any remainder layers to last wildcard IPU
+                    layers_per_ipu[wildcard_idxs[-1]] += remainder
+
+            elif len(layers_per_ipu) != ipus_per_replica:
+                raise IncompatibleIPUConfigError(
+                    "layers_per_ipu has non-default value set, but its length does not match ipus_per_replica. "
+                    f"layers_per_ipu={layers_per_ipu}, ipus_per_replica={ipus_per_replica}. "
                 )
-        return self._wrapped_encoder
-
-    def prepare_inputs_for_generation(self, input_ids: torch.LongTensor, **kwargs) -> Dict[str, Any]:
-        inputs = super().prepare_inputs_for_generation(input_ids, **kwargs)
-        return {k: v for k, v in inputs.items() if k in signature(self._forward_for_generate).parameters}
+            # no wildcards used
+            elif sum(layers_per_ipu) != target_number_of_layers:
+                raise IncompatibleIPUConfigError(
+                    "layers_per_ipu does not define the correct number of layers for the current model."
+                    " The current IPU Config specifies IPU assignments for "
+                    f"{sum(layers_per_ipu)} layers but there are {target_number_of_layers} layers "
+                    f"in the model. layers_per_ipu={layers_per_ipu}"
+                )
+    return layers_per_ipu
 
 
-def get_layer_ipu(layers_per_ipu: List[int], target_number_of_layers: Optional[Union[int, List]] = None):
+def split_encoder_decoder_ipu_config(
+    ipu_config: IPUConfig, num_encoder_layers: int, num_decoder_layers: int
+) -> List[IPUConfig]:
+    """
+    Given an `ipu_config` for an entire encoder-decoder model and the number of encoder and decoder layers,
+    this function will split the `ipu_config` into two separate configs:
+      `encoder_ipu_config` and `decoder_ipu_config`.
+    It will split the `ipu_config.layers_per_ipu` into two given the inputted numbers of encoder and decoder
+    layers.
+
+    Example:
+    ```
+    >> ipu_config = IPUConfig(layers_per_ipu=[12, 12], ipus_per_replica=2)
+    >> encoder_ipu_config, decoder_ipu_config = split_encoder_decoder_ipu_config(ipu_config, 12, 12)
+
+    >> encoder_ipu_config
+    => IPUConfig(layers_ler_ipu=[12], ipus_per_replica=1)
+
+    >> decoder_ipu_config
+    => IPUConfig(layers_ler_ipu=[12], ipus_per_replica=1)
+    ```
+
+    Args:
+        ipu_config: The `IPUConfig` for the the whole encoder-decoder model
+        num_encoder_layers: Number of encoder layers in the model
+        num_decoder_layers: Number of decoder layers in the model
+
+    Returns:
+        encoder_ipu_config, decoder_ipu_config
+    """
+    # Need at least two IPUs to do the split
+    if ipu_config.ipus_per_replica < 2:
+        raise IncompatibleIPUConfigError(
+            "Need ipus_per_replica of at least 2 to split ipu_config into encoder and decoder configs"
+        )
+
+    ipu_configs = {name: copy.deepcopy(ipu_config) for name in ["encoder", "decoder"]}
+
+    # Split layers_per_ipu between the given num layers
+    layers_per_ipu = _expand_layers_per_ipu_wildcard(ipu_config, num_encoder_layers + num_decoder_layers)
+    cumsum = [sum(layers_per_ipu[: i + 1]) for i in range(len(layers_per_ipu))]
+    try:
+        cut = [i + 1 for i, c in enumerate(cumsum) if c == num_encoder_layers]
+        # Choose the cut index that's the highest power of 2
+        cut = max([num for num in cut if num & (num - 1) == 0])
+    except:
+        raise IncompatibleIPUConfigError(
+            f"Unable to find valid split of ipu_config.layers_per_ipu\n"
+            "Arguments: \n"
+            f"\tipu_config.layers_per_ipu={ipu_config.layers_per_ipu}\n"
+            f"\tnum_encoder_layers={num_encoder_layers}\n"
+            f"\tnum_decoder_layers={num_decoder_layers}\n"
+            "Possible causes: \n"
+            "Encoder and decoder layers cannot be placed on the same IPUs.\n"
+            "The encoder and decoder layers_per_ipu splits each need a number of devices that's a power of 2."
+        )
+    ipu_configs["encoder"].layers_per_ipu = layers_per_ipu[:cut]
+    ipu_configs["decoder"].layers_per_ipu = layers_per_ipu[cut:]
+
+    # Modify the ipus_per_replica
+    ipu_configs["encoder"].ipus_per_replica = len(ipu_configs["encoder"].layers_per_ipu)
+    ipu_configs["decoder"].ipus_per_replica = len(ipu_configs["decoder"].layers_per_ipu)
+
+    # Split matmul_proportion between the given num layers
+    matmul_proportion = ipu_config.matmul_proportion
+    if isinstance(matmul_proportion, list):
+        ipu_configs["encoder"].matmul_proportion = matmul_proportion[:cut]
+        ipu_configs["decoder"].matmul_proportion = matmul_proportion[cut:]
+
+    return ipu_configs.values()
+
+
+def get_layer_ipu(ipu_config: IPUConfig, target_number_of_layers: Optional[Union[int, List]] = None) -> List[int]:
+    layers_per_ipu = _expand_layers_per_ipu_wildcard(ipu_config, target_number_of_layers)
+
     # List of the IPU Id for each encoder layer
     layer_ipu: List[int] = []
     for ipu, n_layers in enumerate(layers_per_ipu):
         layer_ipu += [ipu] * n_layers
-    if target_number_of_layers is not None:
-        if not isinstance(target_number_of_layers, (int, float)):
-            target_number_of_layers = len(target_number_of_layers)
-        if len(layer_ipu) < target_number_of_layers:
-            raise ValueError(
-                "layers_per_ipu does not define enough layers for the current model."
-                " The current IPU Config specifies IPU assignments for "
-                f"{len(layer_ipu)} layers but there are {target_number_of_layers} layers "
-                f"in the model. layers_per_ipu={layers_per_ipu}"
-            )
+
     return layer_ipu
 
 
@@ -293,11 +410,13 @@ class SerializedEmbedding(nn.Module):
         # Num embeddings should be divisible by the serialization factor
         assert self.num_embeddings % self.serialization_factor == 0
         self.split_size = self.num_embeddings // self.serialization_factor
+
+        freeze = not embedding.weight.requires_grad
         self.split_embeddings = nn.ModuleList(
             [
                 nn.Embedding.from_pretrained(
                     embedding.weight[i * self.split_size : (i + 1) * self.split_size, :].detach(),
-                    freeze=False,
+                    freeze=freeze,
                     padding_idx=embedding.padding_idx if i == 0 else None,
                 )
                 for i in range(self.serialization_factor)
@@ -312,7 +431,11 @@ class SerializedEmbedding(nn.Module):
         Returns:
             `nn.Embedding` layer
         """
-        return nn.Embedding.from_pretrained(torch.vstack([l.weight for l in self.split_embeddings]), padding_idx=0)
+
+        freeze = not self.split_embeddings[0].weight.requires_grad
+        return nn.Embedding.from_pretrained(
+            torch.vstack([l.weight for l in self.split_embeddings]), padding_idx=0, freeze=freeze
+        )
 
     def forward(self, indices):
         # iterate through the splits
@@ -415,8 +538,18 @@ class SharedEmbedding(nn.Module):
         #     decoder_inputs_embeds = self.shared(decoder_input_ids) * decoder_embed_scale
         # combined, n1, n2 = self._combine_inputs(input_ids, decoder_input_ids)
         # encoder_inputs_embeds, decoder_inputs_embeds = self._separate_inputs(self.shared(combined), n1, n2)
-        idx, combined = self._combine_inputs(input_ids, decoder_input_ids)
-        encoder_inputs_embeds, decoder_inputs_embeds = self._separate_inputs(idx, self.shared(combined))
+        encoder_inputs_embeds, decoder_inputs_embeds = None, None
+        if input_ids is None:
+            # call on decoder_input_ids only
+            decoder_inputs_embeds = self.shared(decoder_input_ids)
+        elif decoder_input_ids is None:
+            # call on input_ids only
+            encoder_inputs_embeds = self.shared(input_ids)
+        else:
+            # Call on the combined case
+            # This case is assuming input_ids and decoder_input_ids are not None
+            idx, combined = self._combine_inputs(input_ids, decoder_input_ids)
+            encoder_inputs_embeds, decoder_inputs_embeds = self._separate_inputs(idx, self.shared(combined))
 
         if encoder_embed_scale:
             encoder_inputs_embeds = encoder_inputs_embeds * encoder_embed_scale
