@@ -25,15 +25,15 @@ from transformers import T5ForConditionalGeneration
 from transformers.modeling_outputs import BaseModelOutput, Seq2SeqLMOutput
 from transformers.models.t5.modeling_t5 import __HEAD_MASK_WARNING_MSG, T5Block, T5Stack
 
-from ...generation_utils import IPUGenerationMixin
+from ...generation_utils import IPUGenerationMixin, _IndexedInputLinear
 from ...modeling_utils import (
-    GenerationMethodsMixin,
     PipelineMixin,
     SerializedLinear,
     SharedEmbedding,
     get_layer_ipu,
     recomputation_checkpoint,
     register,
+    split_encoder_decoder_ipu_config,
 )
 
 
@@ -56,7 +56,6 @@ class CustomT5Block(T5Block):
         output_attentions=False,
         return_dict=True,
     ):
-
         if past_key_value is not None:
             if not self.is_decoder:
                 logger.warning("`past_key_values` is passed to the encoder. Please make sure this is intended.")
@@ -153,9 +152,7 @@ class CustomT5Stack(T5Stack):
 
 
 @register(T5ForConditionalGeneration)
-class PipelinedT5ForConditionalGeneration(
-    GenerationMethodsMixin, T5ForConditionalGeneration, PipelineMixin, IPUGenerationMixin
-):
+class PipelinedT5ForConditionalGeneration(T5ForConditionalGeneration, PipelineMixin, IPUGenerationMixin):
     @property
     def is_encoder_and_decoder_embeddings_computation_shared(self):
         return isinstance(self.shared, SharedEmbedding)
@@ -217,7 +214,7 @@ class PipelinedT5ForConditionalGeneration(
             if not restore:
                 self.lm_scale_modifier /= emb_scaling
 
-    def parallelize(self):
+    def parallelize(self, for_generation=False):
         """
         Transform the model to run in an IPU pipeline.
         - Adds pipeline stages to the model
@@ -229,6 +226,7 @@ class PipelinedT5ForConditionalGeneration(
         model = PipelinedT5ForConditionalGeneration(config).parallelize().half()
         ```
         """
+        PipelineMixin.parallelize(self)
 
         logger.info("-------------------- Device Allocation --------------------")
         logger.info("Embedding  --> IPU 0")
@@ -261,12 +259,25 @@ class PipelinedT5ForConditionalGeneration(
         for block in self.decoder.block:
             block.__class__ = CustomT5Block
 
-        number_of_layers = len(self.encoder.block) + len(self.decoder.block)
-        layer_ipu = get_layer_ipu(self.ipu_config.layers_per_ipu, number_of_layers)
-        for index, layer in enumerate(self.encoder.block):
-            ipu = layer_ipu[index]
+        num_encoder_layers = len(self.encoder.block)
+        num_decoder_layers = len(self.decoder.block)
+
+        if for_generation:
+            # If running for text generation we split the IPU config into two configs
+            # because we run the encoder and decoder as separate Poplar executors.
+            ipu_configs = split_encoder_decoder_ipu_config(self.ipu_config, num_encoder_layers, num_decoder_layers)
+            self.encoder_ipu_config, self.decoder_ipu_config = ipu_configs
+            encoder_layer_ipu = get_layer_ipu(self.encoder_ipu_config, num_encoder_layers)
+            decoder_layer_ipu = get_layer_ipu(self.decoder_ipu_config, num_decoder_layers)
+        else:
+            number_of_layers = num_encoder_layers + num_decoder_layers
+            layer_ipu = get_layer_ipu(self.ipu_config, number_of_layers)
+            encoder_layer_ipu = layer_ipu[:num_encoder_layers]
+            decoder_layer_ipu = layer_ipu[num_encoder_layers:]
+
+        for index, (layer, ipu) in enumerate(zip(self.encoder.block, encoder_layer_ipu)):
             if self.ipu_config.recompute_checkpoint_every_layer and index != self.config.num_layers - 1:
-                recomputation_checkpoint(layer)
+                self._hooks.append(recomputation_checkpoint(layer))
             self.encoder.block[index] = poptorch.BeginBlock(layer, f"Encoder{index}", ipu_id=ipu)
             logger.info(f"Encoder {index:<2} --> IPU {ipu}")
 
@@ -274,11 +285,9 @@ class PipelinedT5ForConditionalGeneration(
             self.encoder.final_layer_norm, "Encoder Stack Final LayerNorm", ipu_id=ipu
         )
 
-        shift = len(self.encoder.block)
-        for index, layer in enumerate(self.decoder.block):
-            ipu = layer_ipu[index + shift]
+        for index, (layer, ipu) in enumerate(zip(self.decoder.block, decoder_layer_ipu)):
             if self.ipu_config.recompute_checkpoint_every_layer and index != self.config.num_layers - 1:
-                recomputation_checkpoint(layer)
+                self._hooks.append(recomputation_checkpoint(layer))
             self.decoder.block[index] = poptorch.BeginBlock(layer, f"Decoder{index}", ipu_id=ipu)
             logger.info(f"Decoder {index:<2} --> IPU {ipu}")
 
@@ -311,6 +320,9 @@ class PipelinedT5ForConditionalGeneration(
         for block in self.decoder.block:
             block.__class__ = T5Block
 
+        if self.lm_head.__class__ == _IndexedInputLinear:
+            self.lm_head = self.lm_head.wrapped_linear
+
         if self.ipu_config.embedding_serialization_factor > 1:
             old_lm_head = nn.Linear(
                 self.config.d_model,
@@ -437,6 +449,7 @@ class PipelinedT5ForConditionalGeneration(
             return Seq2SeqLMOutput(
                 loss=loss,
             )
+
         return Seq2SeqLMOutput(
             loss=loss,
             logits=lm_logits,
