@@ -17,8 +17,7 @@ import functools
 import numpy as np
 import torch
 
-import poptorch
-from transformers.utils import logging
+from optimum.utils import logging
 
 
 logger = logging.get_logger(__name__)
