@@ -65,7 +65,3 @@ class IPUSeq2SeqTrainingArguments(IPUTrainingArguments):
 
     def __post_init__(self):
         super().__post_init__()
-        if self.predict_with_generate:
-            raise NotImplementedError(
-                "--predict_with_generate is not supported on IPUs yet, please run your evaluation on another device or set predict_with_generate to False"
-            )
